// Copyright  2002 - 2007 DataDirect Technologies Corp. All Rights Reserved.
using System;
using System.Data;
using System.Data.Common;
using System.Security.Permissions;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using DDTek.SQLServer;

namespace DDTek.EnterpriseLibrary.Data.SQLServer {

	[SQLServerPermission(SecurityAction.Demand)]
#if ENTLIB41
	[DatabaseAssembler(typeof(SQLServerDatabaseAssembler))]
#endif
	public class SQLServerDatabase : Database {

		/// <summary>
		/// Initializes a new instance of the <see cref="SQLServerDatabase"/> class with a connection string.
		/// </summary>
		/// <param name="connectionString">The connection string.</param>
		public SQLServerDatabase(string connectionString)
			: base(connectionString, SQLServerFactory.Instance) {

		}

#if ENTLIB50

		public override bool  SupportsParemeterDiscovery {
			
			get { return true; }
		}
#endif

		protected override void DeriveParameters(DbCommand discoveryCommand) {

			SQLServerCommandBuilder.DeriveParameters((SQLServerCommand)discoveryCommand);
		}

		/// <summary>
		/// Returns the starting index for parameters in a command.
		/// </summary>
		/// <returns>The starting index for parameters in a command.</returns>
		protected override int UserParametersStartIndex() {

			return 1;
		}

		/// <devdoc>
		/// Listens for the RowUpdate event on a dataadapter to support UpdateBehavior.Continue
		/// </devdoc>
		private void OnSQLServerRowUpdated(object sender, SQLServerRowUpdatedEventArgs rowThatCouldNotBeWritten) {

			if (rowThatCouldNotBeWritten.RecordsAffected == 0) {

				if (rowThatCouldNotBeWritten.Errors != null) {

					rowThatCouldNotBeWritten.Row.RowError = Resources.ExceptionMessageUpdateDataSetRowFailure;
					rowThatCouldNotBeWritten.Status = UpdateStatus.SkipCurrentRow;
				}
			}
		}

		/// <summary>
		/// Sets the RowUpdated event for the data adapter.
		/// </summary>
		/// <param name="adapter">The <see cref="DbDataAdapter"/> to set the event.</param>
		protected override void SetUpRowUpdatedEvent(DbDataAdapter adapter) {

			((SQLServerDataAdapter)adapter).RowUpdated += new SQLServerRowUpdatedEventHandler(OnSQLServerRowUpdated);
		}

		/// <summary>
		/// Determines if the number of parameters in the command matches the array of parameter values.
		/// </summary>
		/// <param name="command">The <see cref="DbCommand"/> containing the parameters.</param>
		/// <param name="values">The array of parameter values.</param>
		/// <returns><see langword="true"/> if the number of parameters and values match; otherwise, <see langword="false"/>.</returns>
		protected override bool SameNumberOfParametersAndValues(DbCommand command, object[] values) {

			// SQLServer procedures have an implicit return value that DeriveParameters will include
			int returnParameterCount = 1;
			int numberOfParametersToStoredProcedure = command.Parameters.Count - returnParameterCount;
			int numberOfValuesProvidedForStoredProcedure = values.Length;

			return (numberOfParametersToStoredProcedure == numberOfValuesProvidedForStoredProcedure);
		}
	}
}
