// Copyright  2002 - 2007 DataDirect Technologies Corp. All Rights Reserved.
using System;
using System.Data;
using System.Data.Common;
using System.Security.Permissions;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using DDTek.Sybase;

namespace DDTek.EnterpriseLibrary.Data.Sybase {

	[SybasePermission(SecurityAction.Demand)]
#if ENTLIB41
	[DatabaseAssembler(typeof(SybaseDatabaseAssembler))]
#endif
	public class SybaseDatabase : Database {

		/// <summary>
		/// Initializes a new instance of the <see cref="SybaseDatabase"/> class with a connection string.
		/// </summary>
		/// <param name="connectionString">The connection string.</param>
		public SybaseDatabase(string connectionString)
			: base(connectionString, SybaseFactory.Instance) {

		}

#if ENTLIB50

		public override bool SupportsParemeterDiscovery {

			get { return true; }
		}
#endif

		protected override void DeriveParameters(DbCommand discoveryCommand) {

			SybaseCommandBuilder.DeriveParameters((SybaseCommand)discoveryCommand);
		}

		/// <devdoc>
		/// Listens for the RowUpdate event on a dataadapter to support UpdateBehavior.Continue
		/// </devdoc>
		private void OnSybaseRowUpdated(object sender, SybaseRowUpdatedEventArgs rowThatCouldNotBeWritten) {

			if (rowThatCouldNotBeWritten.RecordsAffected == 0) {

				if (rowThatCouldNotBeWritten.Errors != null) {

					rowThatCouldNotBeWritten.Row.RowError = Resources.ExceptionMessageUpdateDataSetRowFailure;
					rowThatCouldNotBeWritten.Status = UpdateStatus.SkipCurrentRow;
				}
			}
		}

		/// <summary>
		/// Sets the RowUpdated event for the data adapter.
		/// </summary>
		/// <param name="adapter">The <see cref="DbDataAdapter"/> to set the event.</param>
		protected override void SetUpRowUpdatedEvent(DbDataAdapter adapter) {

			((SybaseDataAdapter)adapter).RowUpdated += new SybaseRowUpdatedEventHandler(OnSybaseRowUpdated);
		}
	}
}
